import os
import sys
import json

import utils

from datetime import datetime
from experiments import *

RESULTS_DIR = os.path.join(os.path.dirname(__file__), "results")
DATA_DIR    = os.path.join(os.path.dirname(__file__), "data")

EXPERIMENTS = {
    "http_request" : http_request.HTTPRequestExperiment,
    "tcp_connect" : tcp_connect.TCPConnectExperiment,
    "turkey"       : turkey.TurkeyExperiment
}

def get_result_file():
    result_file = "result-%s.json" % (datetime.now().isoformat())
    return os.path.join(RESULTS_DIR, result_file)

def get_input_file(experiment_name):
    input_file = "%s.txt" % (experiment_name)
    return os.path.join(DATA_DIR, input_file)

def run():
    if not os.path.exists(RESULTS_DIR):
        print "Creating results directory in %s" % (RESULTS_DIR)
        os.makedirs(RESULTS_DIR)

    result_file = get_result_file()
    result_file = open(result_file, "w")
    results = {}

    for name, exp in EXPERIMENTS.items():
        input_file = get_input_file(name)

        if not os.path.isfile(input_file):
            print "Input file for %s does not exist!" % name
            return

        print "Reading input from %s" % (input_file)
        input_file = open(input_file)

        exp = exp(input_file)
        print "Running %s test." % (name)
        exp.run()

        input_file.close()

        results[name] = exp.results

    json.dump(results, result_file)
    result_file.close()

    print "All experiments over. Check results."

if __name__ == "__main__":
    run()



import httplib

def get_request(host, path="/", headers=None, ssl=False):
    request  = {
        "host"  : host,
        "path"  : path,
        "method": "GET"
    }

    response = {}

    try:
        if ssl:
            conn = httplib.HTTPSConnection(host)
        else:
            conn = httplib.HTTPConnection(host)

        if headers:
            conn.request("GET", path, headers=headers)
        else:
            conn.request("GET", path)

        resp = conn.getresponse()
        response["status"] = resp.status
        response["reason"] = resp.reason

        headers = dict(resp.getheaders())
        response["headers"] = headers

        body = resp.read()
        response["body"] = body

        conn.close()
    except Exception as err:
        response["failure"] = str(err)

    result = {
        "response" : response,
        "request"  : request
    }

    return result



import centinel.utils.http as http
import centinel.utils.dnslib as dns

GOOGLE_DNS     = "8.8.8.8"
SEARCH_STRING  = "home network testbed will appear at"

class TurkeyExperiment:
    name = "turkey"

    def __init__(self, input_file):
        self.input_file = input_file
        self.results = []
        self.host = "twitter.com"
        self.path = "/feamster/status/452889624541921280"

    def run(self):
        ips = dns.get_ips(self.host)
        blocked_ips = filter(self.is_blocked, ips)

        if not blocked_ips:
            return

        # let's try using Google's nameserver
        ips = dns.get_ips(self.host, nameserver=GOOGLE_DNS)
        blocked_ips = filter(self.is_blocked, ips)

        if not blocked_ips:
            return

    def is_blocked(self, ip):
        headers = {
            "Host" : self.host
        }

        result = http.get_request(ip, self.path, headers, ssl=True)

        blocked = SEARCH_STRING not in result["response"]["body"]
        result["blocked"] = blocked

        self.results.append(result)

        return blocked



import dns.resolver

def get_ips(host, nameserver=None, record="A"):
    resolver = dns.resolver.Resolver()

    if nameserver:
        resolver.nameservers = [nameserver]

    #XXX: ipv6?
    answers = resolver.query(host, record)

    ips = [rdata.address for rdata in answers]

    return ips



import socket

class TCPConnectExperiment:
    name = "tcp_connect"

    def __init__(self, input_file):
        self.input_file = input_file
        self.results = []
        self.host = None
        self.port = None

    def run(self):
        for line in self.input_file:
            self.host, self.port = line.strip().split(' ')
            self.tcp_connect()

    def tcp_connect(self):
        result = {
            "host" : self.host,
            "port" : self.port
        }

        try:
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            sock.connect((self.host, int(self.port)))
            sock.close()
            result["success"] = "true"
        except Exception as err:
            result["failure"] = str(err)

        self.results.append(result)



import centinel.utils.http as http

class HTTPRequestExperiment:
    name = "http_request"

    def __init__(self, input_file):
        self.input_file  = input_file
        self.results = []
        self.host = None
        self.path = "/"

    def run(self):
        for line in self.input_file:
            self.host = line.strip()
            self.http_request()

    def http_request(self):
        result = http.get_request(self.host, self.path)
        self.results.append(result)



import os
from setuptools import setup, find_packages

DESCRIPTION = """\
Centinel is a tool used to detect network interference and internet censorship."""

setup(
    name = "centinel",
    version = "0.1.1",
    author = "Sathyanarayanan Gunasekaran",
    author_email = "gsathya@gatech.edu",
    description = DESCRIPTION,
    license = "MIT",
    keywords = "censorship network interference",
    url = "https://www.github.com/projectbismakr/centinel",
    packages = ["centinel", "centinel.experiments", "centinel.utils"],
    install_requires = ["dnspython >= 1.11.0"],
    include_package_data = True,
    entry_points = {
        'console_scripts': ['centinel=centinel.centinel:run']
    },
)
